@@ -1,22 +1,22 @@
 /* Command table manipulation functions and vectors.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1993-1998 Sebastiano Vigna 
+   Copyright (C) 1999-2015 Todd M. Lewis and Sebastiano Vigna
 
-	This file is part of ne, the nice editor.
+   This file is part of ne, the nice editor.
 
-	This library is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 3 of the License, or (at your
-	option) any later version.
+   This library is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
-	This library is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-	for more details.
+   This library is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
 
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, see <http://www.gnu.org/licenses/>.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 
 #include "ne.h"
@@ -163,6 +163,7 @@ static const command commands[ACTION_COUNT] = {
 	{ NAHL(REQUESTORDER  ),                           IS_OPTION                                   },
 	{ NAHL(RIGHTMARGIN   ),                           IS_OPTION                                   },
 	{ NAHL(SAVE          ), NO_ARGS                                                               },
+	{ NAHL(SAVEALL       ), NO_ARGS                                                               },
 	{ NAHL(SAVEAS        ),           ARG_IS_STRING                                               },
 	{ NAHL(SAVEAUTOPREFS ), NO_ARGS                                                               },
 	{ NAHL(SAVECLIP      ),           ARG_IS_STRING                                               },
@@ -201,14 +202,13 @@ static const command commands[ACTION_COUNT] = {
 
 
 /* Checks whether the command line m starts with the command c. Return 0 on
-	success, non-zero on failure. */
-
-int cmdcmp(const unsigned char *c, const unsigned char *m) {
+   success, non-zero on failure. */
 
+int cmdcmp(const char *c, const char *m) {
 	assert(c != NULL);
 	assert(m != NULL);
 
-	while (*c && ascii_up_case[*c] == ascii_up_case[*m]) {
+	while (*c && ascii_up_case[*(unsigned char *)c] == ascii_up_case[*(unsigned char *)m]) {
 		c++;
 		m++;
 	}
@@ -221,68 +221,57 @@ int cmdcmp(const unsigned char *c, const unsigned char *m) {
 
 
 /* This table *can* have conflicts, so that we can keep its size much
-	smaller. */
+   smaller. */
 
 static macro_desc *macro_hash_table[MACRO_HASH_TABLE_SIZE];
 
 
 /* This is the command name hashing function.We consider only the 5 least
-	significant bits because they are the bits which distinguish characters,
-	independently of their case. We are not interested in strings which contain
-	non-alphabetical characters, because they will certainly generate an error
-	(the only exception notably being "R1"). We should subtract 1 to s[i], but
-	this doesn't seem to produce any improvement. hash_macro() act as hash(),
-	but uses MACRO_HASH_TABLE_SIZE for its modulo. */
+   significant bits because they are the bits which distinguish characters,
+   independently of their case. We are not interested in strings which contain
+   non-alphabetical characters, because they will certainly generate an error
+   (the only exception notably being "R1"). We should subtract 1 to s[i], but
+   this doesn't seem to produce any improvement. hash_macro() act as hash(),
+   but uses MACRO_HASH_TABLE_SIZE for its modulo. */
 
 
-static int hash_cmd(const unsigned char * const s, int len) {
+static int hash_cmd(const char * const s, int len) {
 	int h = -1;
-
-	while(len-- != 0) h = (h * 31 + ascii_up_case[s[len]]) % HASH_TABLE_SIZE;
+	while(len-- != 0) h = (h * 31 + ascii_up_case[(unsigned char)s[len]]) % HASH_TABLE_SIZE;
 	return (h + HASH_TABLE_SIZE) % HASH_TABLE_SIZE;
 }
 
 
-static int hash_macro(const unsigned char * const s, int len) {
+static int hash_macro(const char * const s, int len) {
 	int h = -1;
-
-	while(len-- != 0) h = (h * 31 + ascii_up_case[s[len]]) % MACRO_HASH_TABLE_SIZE;
+	while(len-- != 0) h = (h * 31 + ascii_up_case[(unsigned char)s[len]]) % MACRO_HASH_TABLE_SIZE;
 	return (h + MACRO_HASH_TABLE_SIZE) % MACRO_HASH_TABLE_SIZE;
 }
 
 
 
 /* Parses a command line. This function has an interface which is slightly
-	varied with respect to the other functions of ne. In case of a parsing
-	error, an error index *with sign inverted* is passed back. In case parsing
-	succeeds, an (greater or equal to zero) action is returned, and the
-	numerical or string argument is passed in the variables pointed to by
-	num_arg or string_arg, respectively, if they are non-NULL. Otherwise, the
-	argument is not passed back. The string argument is free()able. -1 and NULL
-	denote the lack of an optional numerical or string argument, respectively.
-	NOP is returned on a NOP command, or on a comment line (any line whose first
-	non-space character is a non alphabetic character). Note that the various
-	syntax flags are used here. */
-
-action parse_command_line(const unsigned char * command_line, int * const num_arg, unsigned char ** const string_arg, const int exec_only_options) {
-
-	int h;
-	action a;
-	const unsigned char *p = command_line;
+   varied with respect to the other functions of ne. In case of a parsing
+   error, an error index *with sign inverted* is passed back. In case parsing
+   succeeds, an (greater or equal to zero) action is returned, and the
+   numerical or string argument is passed in the variables pointed to by
+   num_arg or string_arg, respectively, if they are non-NULL. Otherwise, the
+   argument is not passed back. The string argument is free()able. -1 and NULL
+   denote the lack of an optional numerical or string argument, respectively.
+   NOP is returned on a NOP command, or on a comment line (any line whose first
+   non-space character is a non alphabetic character). Note that the various
+   syntax flags are used here. */
 
+int parse_command_line(const char * command_line, int64_t * const num_arg, char ** const string_arg, const bool exec_only_options) {
 	if (num_arg) *num_arg = -1;
 	if (string_arg) *string_arg = NULL;
+	if (!command_line || !*command_line) return NOP_A;
 
-	if (!command_line) return NOP_A;
+	while(isasciispace(*command_line)) command_line++;
+	const char *p = command_line;
 
-	if (!*p) return NOP_A;
-
-	while(isasciispace(*p)) p++;
-
-	command_line = p;
-
-	if (!isalpha(*p)) { /* Comment, treated as NOP. */
-		int len = strlen(p);
+	if (!isalpha((unsigned char)*p)) { /* Comment, treated as NOP. */
+		const int len = strlen(p);
 		if (!(*string_arg = malloc(len + 1))) return -OUT_OF_MEMORY;
 		memcpy(*string_arg, p, len);
 		(*string_arg)[len] = 0;
@@ -291,15 +280,15 @@ action parse_command_line(const unsigned char * command_line, int * const num_ar
 
 	while(*p && !isasciispace(*p)) p++;
 
-	h = hash_cmd(command_line, p - command_line);
-
+	const int h = hash_cmd(command_line, p - command_line);
+	action a;
 	if ((a = hash_table[h]) && !cmdcmp(commands[--a].name, command_line)
 		|| (a = short_hash_table[h]) && !cmdcmp(commands[--a].short_name, command_line)) {
 
 		while(isasciispace(*p)) p++;
 
 		if (!(*p && (commands[a].flags & NO_ARGS))) {
-			if (!*p || (commands[a].flags & ARG_IS_STRING) || isxdigit(*p) || *p == 'x' || *p =='X') {
+			if (!*p || (commands[a].flags & ARG_IS_STRING) || isxdigit((unsigned char)*p) || *p == 'x' || *p =='X') {
 				if ((commands[a].flags & IS_OPTION) || !exec_only_options) {
 					if (*p) {
 						if ((commands[a].flags & ARG_IS_STRING) && string_arg) {
@@ -318,7 +307,7 @@ action parse_command_line(const unsigned char * command_line, int * const num_ar
 						}
 						else if (num_arg) {
 							char *q;
-							*num_arg = strtol(p, &q, 0);
+							*num_arg = strtoll(p, &q, 0);
 							if (*q && !isasciispace(*q)) return -NOT_A_NUMBER;
 						}
 					}
@@ -333,25 +322,23 @@ action parse_command_line(const unsigned char * command_line, int * const num_ar
 		D(fprintf(stderr,"parse_command error: Has no argument.\n");)
 		return -HAS_NO_ARGUMENT;
 	}
-   D(fprintf(stderr,"parse_command error: No such command.\n");)
+	D(fprintf(stderr,"parse_command error: No such command.\n");)
 	return -NO_SUCH_COMMAND;
 }
 
 
 /* Parses and executes a command line. Standard error codes are returned. If
-	the search for a standard command fails, we try to execute a macro in ~/.ne
-	with the same name. */
-
-int execute_command_line(buffer *b, const unsigned char *command_line) {
-
-	int n, a, len = strlen(command_line);
-	encoding_type encoding = detect_encoding(command_line, len);
-	unsigned char *p;
+   the search for a standard command fails, we try to execute a macro in ~/.ne
+   with the same name. */
 
+int execute_command_line(buffer *b, const char *command_line) {
+	encoding_type encoding = detect_encoding(command_line, strlen(command_line));
 	if (b->encoding != ENC_ASCII && encoding != ENC_ASCII && b->encoding != encoding) return INCOMPATIBLE_COMMAND_ENCODING;
 
+	int64_t n;
+	int a;
+	char *p;
 	if ((a = parse_command_line(command_line, &n, &p, b->exec_only_options)) >= 0) return do_action(b, a, n, p);
-
 	a = -a;
 
 	if ((a == NO_SUCH_COMMAND) && (a = execute_macro(b, command_line)) == CANT_OPEN_MACRO) a = NO_SUCH_COMMAND;
@@ -362,7 +349,7 @@ int execute_command_line(buffer *b, const unsigned char *command_line) {
 
 
 /* Allocates a macro descriptor. It does not allocate the internal character
-	stream, which has to be allocated and stuffed in separately. */
+   stream, which has to be allocated and stuffed in separately. */
 
 macro_desc *alloc_macro_desc(void) {
 	return calloc(1, sizeof(macro_desc));
@@ -386,18 +373,17 @@ void free_macro_desc(macro_desc *md) {
 
 
 /* Here we record an action in a character stream. The action name is expanded
-	in a short or long name, depending on the value of the verbose parameter.  A
-	numerical or string argument are expanded and copied, too. If the command
-	should not be recorded (for instance, ESCAPE_A) we return. */
+   in a short or long name, depending on the value of the verbose parameter.  A
+   numerical or string argument are expanded and copied, too. If the command
+   should not be recorded (for instance, ESCAPE_A) we return. */
 
-void record_action(char_stream *cs, action a, int c, unsigned char *p, int verbose) {
+void record_action(char_stream *cs, action a, int64_t c, const char *p, bool verbose) {
+	if (commands[a].flags & DO_NOT_RECORD) return;
 
 	char t[MAX_INT_LEN + 2];
 
-	if (commands[a].flags & DO_NOT_RECORD) return;
-	
 	/* NOP_A is special; it may actually be a comment.
-	   Blank lines and real NOPs are recorded as blank lines. */
+		Blank lines and real NOPs are recorded as blank lines. */
 	if (a == NOP_A) {
 		if (p && *p) add_to_stream(cs, p, strlen(p) + 1);
 		else add_to_stream(cs, "", 1);
@@ -407,9 +393,8 @@ void record_action(char_stream *cs, action a, int c, unsigned char *p, int verbo
 	if (verbose) add_to_stream(cs, commands[a].name, strlen(commands[a].name));
 	else add_to_stream(cs, commands[a].short_name, strlen(commands[a].short_name));
 
-
 	if (c >= 0) {
-		sprintf(t, " %d", c);
+		sprintf(t, " %" PRId64, c);
 		add_to_stream(cs, t, strlen(t));
 	}
 	else if (p) {
@@ -425,26 +410,21 @@ void record_action(char_stream *cs, action a, int c, unsigned char *p, int verbo
 
 
 /* A support function for optimize_macro(). It examines a string to see if it
-	is a valid "InsertChar ##" command. If it is, then insertchar_val() returns
-	the character code, otherwise it returns 0. */
+   is a valid "InsertChar ##" command. If it is, then insertchar_val() returns
+   the character code, otherwise it returns 0. */
 
-static int insertchar_val(const unsigned char *p) {
-	int h;
-	const unsigned char *cmd;
-	action a;
-    
+static int insertchar_val(const char *p) {
 	if ( !p || !*p) return 0;
-    
 	while(isasciispace(*p)) p++;
+	const char * const cmd = p;
+
+	if (!isalpha((unsigned char)*p)) return 0;
 
-	cmd = p;
-	
-	if (!isalpha(*p)) return 0;
-	
 	while(*p && !isasciispace(*p)) p++;
-	
-	h = hash_cmd(cmd, p - cmd);
-	
+
+	int h = hash_cmd(cmd, p - cmd);
+
+	action a;
 	if (((a = hash_table[h]) && !cmdcmp(commands[--a].name, cmd)
 		|| (a = short_hash_table[h]) && !cmdcmp(commands[--a].short_name, cmd))	&& a == INSERTCHAR_A) {
 
@@ -455,36 +435,54 @@ static int insertchar_val(const unsigned char *p) {
 	return 0;
 }
 
-/* Looks through the macro stream for consecutive runs of InsertChar commands
-	and replaces them with appropriate InsertString commands. This makes macros
-	much easier to read if and when they have to be edited. Note that if the
-	character inserted by InsertChar is not an ASCII character, then we should
-	leave it as an InsertChar command to maximize portability of the macros. */
+/* Optimizing macros is not safe if there are any subsequent undo commands
+   calls to macros (which may themselves contain undo commands). This function
+   looks through a stream for undo or non-built in commands, and returns false
+   if any are found; returns true otherwise. */
 
-void optimize_macro(char_stream *cs, int verbose) {
-	char *cmd;
-	int pos;
-	int chr;
-	int building = 0;
-	 
+bool vet_optimize_macro_stream(char_stream * const cs, int64_t pos) {
+	int64_t n;
+	int a;
+	char *p;
+
+	while (pos < cs->len ) {
+		if ((a = parse_command_line(&cs->stream[pos], &n, &p, 0)) >= 0) {
+			if (p) free(p);
+			if (a == UNDO_A) return false; /* optimization is not safe */
+		} else {
+			a = -a;
+			if (a == NO_SUCH_COMMAND) return false; /* possibly a macro invocation */
+		}
+		pos += strlen(&cs->stream[pos]) + 1;
+	}
+	return true;
+}
+
+/* Looks through the macro stream for consecutive runs of InsertChar commands
+   and replaces them with appropriate InsertString commands. This makes macros
+   much easier to read if and when they have to be edited. Note that if the
+   character inserted by InsertChar is not an ASCII character, then we should
+   leave it as an InsertChar command to maximize portability of the macros. */
+
+void optimize_macro(char_stream *cs, bool verbose) {
 	if (!cs || !cs->len) return;
 
-	for (pos = 0; pos < cs->len; pos += strlen(&cs->stream[pos]) + 1) {
-		cmd = &cs->stream[pos];
-		chr = insertchar_val(cmd);
-		if (chr < 0x80 && isprint(chr)) {
-			char two[2] = " ";
-			two[0] = chr;
+	int building = 0;
+	bool safe_to_optimize = false;
+
+	for (int64_t pos = 0; pos < cs->len; pos += strlen(&cs->stream[pos]) + 1) {
+		char * const cmd = &cs->stream[pos];
+		const int chr = insertchar_val(cmd);
+		if (chr < 0x80 && isprint(chr) && (safe_to_optimize = vet_optimize_macro_stream(cs,pos))) {
 			delete_from_stream(cs, pos, strlen(cmd) + 1);
+			const char two[2] = { chr };
 			if (building) {
 				building++;
 				insert_in_stream(cs, two, building, 1);
 			}
 			else {
-				const char *insert;
-				int len;
-				insert = verbose ? INSERTSTRING_NAME : INSERTSTRING_ABBREV;
-				len  = strlen(insert);
+				const char * const insert = verbose ? INSERTSTRING_NAME : INSERTSTRING_ABBREV;
+				const int64_t len  = strlen(insert);
 				insert_in_stream(cs, "\"",   pos, 2);   /* Closing quote */
 				insert_in_stream(cs, two,    pos, 1);   /* The character itself */
 				insert_in_stream(cs, " \"",  pos, 2);   /* space and openning quote */
@@ -492,38 +490,35 @@ void optimize_macro(char_stream *cs, int verbose) {
 				building = pos + len + 2;               /* This is where the char is now */
 			}
 		}
-		else {
-			building = 0;
-		}
+		else building = 0;
 	}
 }
-  
+
 
 /* This function is the ultimate goal of this file. It plays a character
-	stream, considering each line as a command line. It polls the global stop
-	variable in order to check for the user interrupting. Note that the macro is
-	duplicated before execution: this is absolutely necessary, for otherwise a
-	call to CloseDoc, Record or UnloadMacros could free() the block of memory
-	which we are executing. */
+   stream, considering each line as a command line. It polls the global stop
+   variable in order to check for the user interrupting. Note that the macro is
+   duplicated before execution: this is absolutely necessary, for otherwise a
+   call to CloseDoc, Record or UnloadMacros could free() the block of memory
+   which we are executing. */
 
 int play_macro(buffer *b, char_stream *cs) {
-
-	int error = OK, len;
-	unsigned char *p, *stream;
-
 	if (!cs) return ERROR;
 
 	/* If len is 0 or 1, the character stream does not contain anything. */
+	const int64_t len = cs->len;
+	if (len < 2) return OK;
 
-	if ((len = cs->len) < 2) return OK;
-
-	if (!(p = stream = malloc(len))) return OUT_OF_MEMORY;
+	char * const stream = malloc(len);
+	if (!stream) return OUT_OF_MEMORY;
+	char * p = stream;
 
 	memcpy(stream, cs->stream, len);
 
-	stop = FALSE;
+	stop = false;
 
 	b->executing_macro = 1;
+	int error = OK;
 	while(!stop && p - stream < len) {	
 #ifdef NE_TEST
 		fprintf(stderr, "%s\n", p); /* During tests, we output to stderr the current command. */
@@ -550,35 +545,32 @@ int play_macro(buffer *b, char_stream *cs) {
 
 
 /* Loads a macro, and puts it in the global macro hash table.  file_part is
-	applied to the name argument before storing it and hashing it.  Note that if
-	the macro can't be opened, we retry prefixing its name with the preferences
-	directory name (~/.ne/). Thus, for instance, all autopreferences file whose
-	name does not conflict with internal commands can be executed transparently
-	just by typing their name. */
+   applied to the name argument before storing it and hashing it.  Note that if
+   the macro can't be opened, we retry prefixing its name with the preferences
+   directory name (~/.ne/). Thus, for instance, all autopreferences file whose
+   name does not conflict with internal commands can be executed transparently
+   just by typing their name. */
 
 
 macro_desc *load_macro(const char *name) {
 
-	int h;
-	char *macro_dir, *prefs_dir;
-	char_stream *cs;
-	macro_desc *md, **m;
-
 	assert(name != NULL);
 
-	if (!(md = alloc_macro_desc())) return NULL;
+	macro_desc * const md = alloc_macro_desc();
+	if (!md) return NULL;
 
-	cs = load_stream(md->cs, name, FALSE, FALSE);
+	char_stream * cs = load_stream(md->cs, name, false, false);
 
+	char *macro_dir, *prefs_dir;
 	if (!cs && (prefs_dir = exists_prefs_dir()) && (macro_dir = malloc(strlen(prefs_dir) + 2 + strlen(name)))) {
 		strcat(strcpy(macro_dir, prefs_dir), name);
-		cs = load_stream(md->cs, macro_dir, FALSE, FALSE);
+		cs = load_stream(md->cs, macro_dir, false, false);
 		free(macro_dir);
 	}
 
-	if (!cs && (prefs_dir = exists_gprefs_dir()) && (macro_dir = malloc(strlen(prefs_dir) + 2 + strlen(name)))) {
-		strcat(strcpy(macro_dir, prefs_dir), name);
-		cs = load_stream(md->cs, macro_dir, FALSE, FALSE);
+	if (!cs && (prefs_dir = exists_gprefs_dir()) && (macro_dir = malloc(strlen(prefs_dir) + 2 + strlen(name) + 7))) {
+		strcat(strcat(strcpy(macro_dir, prefs_dir), "macros/"), name);
+		cs = load_stream(md->cs, macro_dir, false, false);
 		free(macro_dir);
 	}
 
@@ -589,9 +581,9 @@ macro_desc *load_macro(const char *name) {
 		md->cs = cs;
 		md->name = str_dup(file_part(name));
 
-		h = hash_macro(md->name, strlen(md->name));
+		const int h = hash_macro(md->name, strlen(md->name));
 
-		m = &macro_hash_table[h];
+		macro_desc **m = &macro_hash_table[h];
 
 		while(*m) m = &((*m)->next);
 
@@ -610,25 +602,18 @@ macro_desc *load_macro(const char *name) {
 macro list, it is loaded. A standard error code is returned. */
 
 int execute_macro(buffer *b, const char *name) {
-
-	static call_depth = 0;
-	const char *p;
-	macro_desc *md;
-	int h;
+	static int call_depth = 0;
 
 	if (++call_depth > 32) {
 		--call_depth;
 		return MAX_MACRO_DEPTH_EXCEEDED;
 	}
 
-	p = file_part(name);
-
-	h = hash_macro(p, strlen(p));
-
-	md = macro_hash_table[h];
-
-	while(md && cmdcmp(md->name, p)) md = md->next;
+	const char * const p = file_part(name);
+	int h = hash_macro(p, strlen(p));
 
+	macro_desc *md;
+	for(md = macro_hash_table[h]; md && cmdcmp(md->name, p); md = md->next );
 	if (!md) md = load_macro(name);
 
 	assert_macro_desc(md);
@@ -655,17 +640,11 @@ int execute_macro(buffer *b, const char *name) {
 /* Clears up the macro table. */
 
 void unload_macros(void) {
-
-	int i;
-	macro_desc *m, *n;
-
-	for(i = 0; i < MACRO_HASH_TABLE_SIZE; i++) {
-
-		m = macro_hash_table[i];
+	for(int i = 0; i < MACRO_HASH_TABLE_SIZE; i++) {
+		macro_desc *m = macro_hash_table[i];
 		macro_hash_table[i] = NULL;
-
 		while(m) {
-			n = m->next;
+			macro_desc * const n = m->next;
 			free_macro_desc(m);
 			m = n;
 		}
@@ -675,22 +654,21 @@ void unload_macros(void) {
 /* Find any key strokes that currently map to commands[i].name or commands[i].short_name.
    Returns either NULL or a char string that must be freed by the caller. */
 char *find_key_strokes(int c) {
-	int i;
 	char *str=NULL, *p;
-	
-	for (i=0; i<NUM_KEYS; i++) {
+
+	for(int i = 0; i < NUM_KEYS; i++) {
 		if (key_binding[i]) {
 			if (((!strncasecmp(commands[c].short_name,key_binding[i],strlen(commands[c].short_name))) &&
-			     ((!key_binding[i][strlen(commands[c].short_name)]      ) || 
-			      (key_binding[i][strlen(commands[c].short_name)] == ' ')
-			     )
-			    ) || 
-			    ((!strncasecmp(commands[c].name,key_binding[i],strlen(commands[c].name))) &&
-			     ((!key_binding[i][strlen(commands[c].name)]      ) ||
-			      (key_binding[i][strlen(commands[c].name)] == ' ')
-			     )
-			    )
-			   ) {
+				  ((!key_binding[i][strlen(commands[c].short_name)]		) || 
+					(key_binding[i][strlen(commands[c].short_name)] == ' ')
+				  )
+				 ) || 
+				 ((!strncasecmp(commands[c].name,key_binding[i],strlen(commands[c].name))) &&
+				  ((!key_binding[i][strlen(commands[c].name)]		) ||
+					(key_binding[i][strlen(commands[c].name)] == ' ')
+				  )
+				 )
+				) {
 				if (!str) {
 					if (!(str = malloc(14))) return NULL;
 					strcpy(str,"Bound key(s):");
@@ -708,59 +686,72 @@ char *find_key_strokes(int c) {
 }
 
 /* This function helps. The help text relative to the command name pointed to
-	by p is displayed (p can also contain arguments). If p is NULL, the
-	alphabetically ordered list of commands is displayed with the string
-	requester. The help finishes when the user escapes. */
+   by p is displayed (p can also contain arguments). If p is NULL, the
+   alphabetically ordered list of commands is displayed with the string
+   requester. The help finishes when the user escapes. */
 
 void help(char *p) {
+	req_list rl = { .ignore_tab=true };
 
-	action a;
-	char *key_strokes;
-	char **tmphelp;
-	int j, i = 0;
 	D(fprintf(stderr,"Help Called with parm %p.\n",p);)
+	int r = 0;
 	do {
 		print_message("Help: select Command and press Enter, or F1 or Escape or Escape-Escape");
-		if (p || (i = request_strings(command_names, ACTION_COUNT, i, MAX_COMMAND_WIDTH, FALSE)) >= 0) {
-         D(fprintf(stderr,"Help check #2: p=%p, i=%d\n",p,i);)
+		rl.cur_entries = ACTION_COUNT;
+		rl.alloc_entries = 0;
+		rl.max_entry_len = MAX_COMMAND_WIDTH;
+		rl.entries = (char **)command_names;
+		if (p || (r = request_strings(&rl, r)) >= 0) {
+			D(fprintf(stderr,"Help check #2: p=%p, r=%d\n",p,r);)
 			if (p) {
-				for(i = 0; i < strlen(p); i++) if (isasciispace((unsigned char)p[i])) break;
+				for(int i = 0; i < strlen(p); i++) if (isasciispace((unsigned char)p[i])) break;
 
-				i = hash_cmd(p, i);
+				r = hash_cmd(p, r);
 
-				if ((a = hash_table[i]) && !cmdcmp(commands[--a].name, p)
-				|| (a = short_hash_table[i]) && !cmdcmp(commands[--a].short_name, p)) i = a;
-				else i = -1;
+				action a;
+				if ((a = hash_table[r]) && !cmdcmp(commands[--a].name, p)
+				|| (a = short_hash_table[r]) && !cmdcmp(commands[--a].short_name, p)) r = a;
+				else r = -1;
 
 				p = NULL;
 			}
 			else {
-				D(fprintf(stderr,"Gonna parse_command_line(\"%s\",NULL,NULL,FALSE);\n",command_names[i]);)
-				i = parse_command_line(command_names[i], NULL, NULL, FALSE);
-				D(fprintf(stderr,"...and got i=%d\n",i);)
+				D(fprintf(stderr,"Gonna parse_command_line(\"%s\",NULL,NULL,false);\n",command_names[r]);)
+				r = parse_command_line(command_names[r], NULL, NULL, false);
+				D(fprintf(stderr,"...and got r=%d\n",r);)
 			}
-			
-			if (i < 0) {
-				i = 0;
+
+			if (r < 0) {
+				r = 0;
 				continue;
 			}
 
-			assert(i >= 0 && i < ACTION_COUNT);
+			assert(r >= 0 && r < ACTION_COUNT);
 
 			print_message("Help: press Enter, or F1 or Escape or Escape-Escape");
-			if ((key_strokes = find_key_strokes(i)) && (tmphelp = calloc(commands[i].help_len+1, sizeof(char *)))) {
-				tmphelp[0] = (char *)commands[i].help[0];
-				tmphelp[1] = (char *)commands[i].help[1];
+			char *key_strokes, **tmphelp;
+			if ((key_strokes = find_key_strokes(r)) && (tmphelp = calloc(commands[r].help_len+1, sizeof(char *)))) {
+				tmphelp[0] = (char *)commands[r].help[0];
+				tmphelp[1] = (char *)commands[r].help[1];
 				tmphelp[2] = key_strokes;
-				memcpy(&tmphelp[3], &commands[i].help[2], sizeof(char *) * (commands[i].help_len-2)); 
-				if ((j = request_strings((const char * const* const)tmphelp, commands[i].help_len+1, 0, ne_columns, FALSE)) < 0 ) i = j;
+				memcpy(&tmphelp[3], &commands[r].help[2], sizeof(char *) * (commands[r].help_len-2)); 
+				rl.cur_entries = commands[r].help_len+1;
+				rl.alloc_entries = 0;
+				rl.max_entry_len = ne_columns;
+				rl.entries = tmphelp;
+				const int s = request_strings(&rl, 0);
+				if (s < 0) r = s;
 				free(tmphelp);
 			} else {
-				if ((j = request_strings(commands[i].help, commands[i].help_len, 0, ne_columns, FALSE)) < 0 ) i = j;
+				rl.cur_entries = commands[r].help_len;
+				rl.alloc_entries = 0;
+				rl.max_entry_len = ne_columns;
+				rl.entries = (char **)commands[r].help;
+				const int s = request_strings(&rl, 0);
+				if (s < 0) r = s;
 			}
-			if (key_strokes)
-				free(key_strokes);
+			if (key_strokes) free(key_strokes);
 		}
-	} while(i >= 0);
+	} while(r >= 0);
 	draw_status_bar();
 }
