@@ -1,7 +1,7 @@
 /* terminfo emulation through GNU termcap definitions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2015 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -143,8 +143,3 @@ extern char *key_f63;
 
 #define resetterm()
 #define fixterm()
-
-/* These are defined in curses.h. The code relies on them. */
-
-#define TRUE 1
-#define FALSE 0
