@@ -1,22 +1,22 @@
 /* Input class, key bindings and upper casing vector definitions.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1993-1998 Sebastiano Vigna 
+   Copyright (C) 1999-2015 Todd M. Lewis and Sebastiano Vigna
 
-	This file is part of ne, the nice editor.
+   This file is part of ne, the nice editor.
 
-	This library is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 3 of the License, or (at your
-	option) any later version.
+   This library is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
-	This library is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-	for more details.
+   This library is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
 
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, see <http://www.gnu.org/licenses/>.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 
 #include "ne.h"
@@ -26,10 +26,10 @@ const char *input_class_names[INPUT_CLASS_COUNT] = {
 }; 
 
 /* This vector contains all key bindings. Each entry points to a command line
-	to be executed when the corresponding keystroke is input. The index
-	correspond to the ASCII code, and to the codes defined in keycodes.h for the
-	special keys. Note that it is nonsense to specify a binding for a key whose
-	class is not COMMAND. */
+   to be executed when the corresponding keystroke is input. The index
+   correspond to the ASCII code, and to the codes defined in keycodes.h for the
+   special keys. Note that it is nonsense to specify a binding for a key whose
+   class is not COMMAND. */
 
 #ifndef ALTPAGING
   #define PICK(A,B) A
@@ -76,8 +76,8 @@ const char *key_binding[NUM_KEYS] = {
 	/* ^] */      MATCHBRACKET_ABBREV,
 	/* ^^ */      ADJUSTVIEW_ABBREV,
 	/* ^_ */      FINDREGEXP_ABBREV,
-       
-   /* These keys map to strings, not commands. 31..126  20..7e */
+
+	/* These keys map to strings, not commands. 31..126  20..7e */
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /* space ! " # $ % & ' ( ) * + , - . /  */
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /*     0 1 2 3 4 5 6 7 8 9 : ; < = > ?  */
 
@@ -85,7 +85,7 @@ const char *key_binding[NUM_KEYS] = {
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /* P Q R S T U V W X Y Z [ \ ] ^ _  */
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /* ` a b c d e f g h i j k l m n o  */
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,        /* p q r s t u v w x y z { | } ~    */
-   DELETECHAR_ABBREV,                         /*  127  7f */
+	DELETECHAR_ABBREV,                         /*  127  7f */
 
 	/* Control-meta-letter bindings (Ctrl-Alt-X)  128..159 80..9f */
 
@@ -140,8 +140,8 @@ const char *key_binding[NUM_KEYS] = {
 	LINEDOWN_ABBREV, 
 	MOVELEFT_ABBREV, 
 	MOVERIGHT_ABBREV, 
-	MOVEINCUP_ABBREV, 
-	MOVEINCDOWN_ABBREV, 
+	PICK(MOVEINCUP_ABBREV,MOVESOL_ABBREV), 
+	PICK(MOVEINCDOWN_ABBREV,MOVEEOL_ABBREV), 
 	PICK(NEXTPAGE_ABBREV,PAGEDOWN_ABBREV),
 	PICK(PREVPAGE_ABBREV,PAGEUP_ABBREV), 
 	LINEDOWN_ABBREV, 
@@ -230,7 +230,7 @@ const char *key_binding[NUM_KEYS] = {
 	/* ESC Ctrl-m */ PLAYONCE_ABBREV, 
 	/* ESC Ctrl-n */ OPENNEW_ABBREV, 
 	/* ESC Ctrl-o */ OPENCLIP_ABBREV,
-	
+
 	/* ESC Ctrl-p */ PARAGRAPH_ABBREV, 
 	/* ESC Ctrl-q */ QUIT_ABBREV, 
 	/* ESC Ctrl-r */ REDO_ABBREV, 
@@ -251,7 +251,7 @@ const char *key_binding[NUM_KEYS] = {
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /* 416..431  1A0..1AF */
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,  /* 432..447  1B0..1BF */
 
-   /* Prefix-simulated META (ESC X)  448..479   1C0..1DF */
+	/* Prefix-simulated META (ESC X)  448..479   1C0..1DF */
 	/* ESC @ */ NULL, 
 	/* ESC A */ MOVESOF_ABBREV, 
 	/* ESC B */ PREVWORD_ABBREV, 
@@ -287,7 +287,7 @@ const char *key_binding[NUM_KEYS] = {
 	/* ESC _ */ NULL,
 
 
-   /* Prefix-simulated META (ESC X)  480..511  1E0..1FF */
+	/* Prefix-simulated META (ESC X)  480..511  1E0..1FF */
 	/* ESC @ */ NULL, 
 	/* ESC a */ MOVESOF_ABBREV, 
 	/* ESC b */ PREVWORD_ABBREV, 
@@ -506,7 +506,7 @@ const char *key_stroke[NUM_KEYS] = {
 	"^A",
 	"^B",
 	"^C",
-	"^D",      
+	"^D",
 	"^E",      
 	"^F",      
 	"^G",      
@@ -537,16 +537,16 @@ const char *key_stroke[NUM_KEYS] = {
 	"^]", 
 	"^^", 
 	"^_", 
-       
-   /* These keys map to strings, not commands. 31..126  20..7e */
+
+	/* These keys map to strings, not commands. 31..126  20..7e */
 	" ", "!", "\"","#", "$", "%", "&", "'", "(", ")", "*", "+", ",", "-", ".", "/",   /* space ! " # $ % & ' ( ) * + , - . /  */
 	"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", ":", ";", "<", "=", ">", "?",   /*     0 1 2 3 4 5 6 7 8 9 : ; < = > ?  */
-   
+
 	"@", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O",   /* @ A B C D E F G H I J K L M N O  */ 
 	"P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "[", "\\","]", "^", "_",   /* P Q R S T U V W X Y Z [ \ ] ^ _  */
 	"`", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o",   /* ` a b c d e f g h i j k l m n o  */
 	"p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "{", "|", "}", "~",        /* p q r s t u v w x y z { | } ~    */
-   "Del",                         /*  127  7f */
+	"Del",                         /*  127  7f */
 
 	/* Control-meta-letter bindings (Ctrl-Alt-X)  128..159 80..9f */
 
@@ -691,7 +691,7 @@ const char *key_stroke[NUM_KEYS] = {
 	"ESC-^M",
 	"ESC-^N",
 	"ESC-^O",
-	
+
 	"ESC-^P",
 	"ESC-^Q",
 	"ESC-^R",
@@ -712,7 +712,7 @@ const char *key_stroke[NUM_KEYS] = {
 	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 416..431  1A0..1AF */
 	"", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "",  /* 432..447  1B0..1BF */
 
-   /* Prefix-simulated META (ESC X)  448..479   1C0..1DF */
+	/* Prefix-simulated META (ESC X)  448..479   1C0..1DF */
 	"ESC-@",
 	"ESC-A",
 	"ESC-B",
@@ -748,7 +748,7 @@ const char *key_stroke[NUM_KEYS] = {
 	"ESC-_",
 
 
-   /* Prefix-simulated META (ESC X)  480..511  1E0..1FF */
+	/* Prefix-simulated META (ESC X)  480..511  1E0..1FF */
 	"ESC-@",
 	"ESC-a",
 	"ESC-b",
