@@ -1,22 +1,22 @@
 /* Terminfo database scanning and keyboard escape sequence matching functions.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1993-1998 Sebastiano Vigna 
+   Copyright (C) 1999-2015 Todd M. Lewis and Sebastiano Vigna
 
-	This file is part of ne, the nice editor.
+   This file is part of ne, the nice editor.
 
-	This library is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 3 of the License, or (at your
-	option) any later version.
+   This library is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
-	This library is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-	for more details.
+   This library is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
 
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, see <http://www.gnu.org/licenses/>.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 
 #include "ne.h"
@@ -41,18 +41,18 @@ order is *inverted* w.r.t. strcmp(). */
 
 
 typedef struct {
-	unsigned char *string;
+	const char *string;
 	int code;
 } term_key;
 
 
 static term_key key[MAX_TERM_KEY];
-static int num_keys = 0;
+static int num_keys;
 
 /* Function to pass to qsort for sorting the key capabilities array. */
 
 static int keycmp(const void *t1, const void *t2) {
-	return -strcmp((char *)((term_key *)t1)->string, (char *)((term_key *)t2)->string);
+	return -strcmp(((term_key *)t1)->string, ((term_key *)t2)->string);
 }
 
 
@@ -61,57 +61,48 @@ position pos return -pos-1 (i.e., always a negative number), otherwise return
 the correct place for insertion of s */
 
 int binsearch(const char * const s) {
-	int l,r,m = 0;
-
 	if (num_keys) {
-		l = 0;
-		r = num_keys - 1;
-		
+		int l = 0, m = 0, r = num_keys - 1;
+
 		while(l <= r) {
 			m = (l + r) / 2;
 			if (is_prefix(s, key[m].string)) return -m - 1;
 			if (strcmp(key[m].string, s) > 0) l = m + 1;
 			else if (strcmp(key[m].string, s) < 0) r = m - 1;
 		}
-		
+
+		return l;
 	}
 	else return 0;
-
-	return l;
 }
 
 
 
 #ifdef DEBUGPRINTF
-void dump_keys(void)
-  {
-    int i;
-    for (i = 0; i < num_keys; i++)
-      {
-        unsigned char *p = key[i].string;
-        fprintf(stderr,"%3d: \"",i);
-        while (*p)
-          {
-            if (isprint(*p)) fprintf(stderr,"%c",*p);
-            else fprintf(stderr,"\\x%02x", *p );
-            p++;
-          }
-        fprintf (stderr,"\"\t-> %d\n", key[i].code );
-      }
-  }
+void dump_keys(void) {
+	 for (int i = 0; i < num_keys; i++) {
+		char *p = key[i].string;
+		fprintf(stderr,"%3d: \"",i);
+		while (*p) {
+			if (isprint(*p)) fprintf(stderr,"%c",*p);
+			else fprintf(stderr,"\\x%02x", *p );
+			p++;
+		}
+		fprintf (stderr,"\"\t-> %d\n", key[i].code );
+	}
+}
 
 #endif
 
 
 /* Sets the first free position in the key capabilities array to the cap_string
-	capability, and increment the first free position counter.
+   capability, and increment the first free position counter.
 */
 
 static void key_set(const char * const cap_string, const int code) {
-
 	if (!cap_string) return;
 
-	key[num_keys].string = (unsigned char *)cap_string;
+	key[num_keys].string = cap_string;
 	key[num_keys].code = code;
 	num_keys++;
 
@@ -119,26 +110,32 @@ static void key_set(const char * const cap_string, const int code) {
 
 
 
-/* key_may_set() sets a key capability string IFF it isn't already
-assigned. It assumes the array is already sorted, and it keeps it
-that way. This is part of the horrible hack to make cursor and
-function keys work on numerous terminals which have broken terminfo
-and termcap entries, or for weak terminal emulators which happen to
-produce well-known sequences. Returns
-   > 0 on success,
-   ==0 if table is full (or no cap_string supplied)
-   < 0 if string is already defined.
- */
-
-
-int key_may_set(const char * const cap_string, const int code) {
-	int pos=0;
-
-   if (num_keys >= MAX_TERM_KEY - 1) return 0;
-	if (!cap_string || (pos = binsearch(cap_string)) < 0) return pos;
+/* key_may_set() maps a key capability string to a key code number.
+   It assumes the array is already sorted, and it keeps it that way.
+   If the code number is positive and the cap_string is already in
+   the key map, no mapping is done. If the code number is negative
+   and the cap_string is already in the key vector, the matching code
+   is replaced with the positive counterpart of the code passed in.
+      This is part of the horrible hack to make cursor and
+   function keys work on numerous terminals which have broken terminfo
+   and termcap entries, or for weak terminal emulators which happen to
+   produce well-known sequences. Returns
+	   > 0 on success,
+	   ==0 if table is full (or no cap_string supplied)
+	   < 0 if string was already defined.   
+*/
+int key_may_set(const char * const cap_string, int code) {
+	int pos = 0;
 
+	if (num_keys >= MAX_TERM_KEY - 1) return 0;
+	if (!cap_string || (pos = binsearch(cap_string)) < 0) {
+		if (code < 0)
+			key[-pos-1].code = -code - 1;
+		return pos;
+	}
+   if (code < 0) code = -code - 1;
 	memmove(key + pos + 1, key + pos, (num_keys - pos) * sizeof *key);
-	key[pos].string = (unsigned char *)cap_string;
+	key[pos].string = cap_string;
 	key[pos].code = code;
 	num_keys++;
 	assert(num_keys < MAX_TERM_KEY);
@@ -157,8 +154,6 @@ almost always greater than or equal to ' '). */
 extern const char meta_prefixed[128][3];
 
 void read_key_capabilities(void) {
-	int i;
-
 	if (!ansi) {
 		/* Cursor movement keys */
 
@@ -288,7 +283,7 @@ void read_key_capabilities(void) {
 		generally in sad shape too, but that's a much larger problem.)  However,
 		certain escape sequences are quite common among large sets of terminals,
 		and so we define the most common ones here.
-		
+
 		key_may_set() won't assign key cap strings if that sequence is already
 		taken, so we shouldn't be doing too much damage if the terminfo or
 		termcap happens to be correct.  */
@@ -334,14 +329,14 @@ void read_key_capabilities(void) {
 	key_may_set("\x1B\x1B", NE_KEY_F(1));
 
 
-   /* More hacking. Function keys are routinely defined wrong on bazillions of
-      systems. This sections codes the F1-F10 keys for vt100, xterms and PCs. I
-      can't believe vendors can ship such buggy termcap/terminfo entries.  This
-      also handles the case of an otherwise limited terminal emulator which
-      happens to produce these sequences for function keys.  */
+	/* More hacking. Function keys are routinely defined wrong on bazillions of
+		systems. This sections codes the F1-F10 keys for vt100, xterms and PCs. I
+		can't believe vendors can ship such buggy termcap/terminfo entries.  This
+		also handles the case of an otherwise limited terminal emulator which
+		happens to produce these sequences for function keys.  */
 
-   /* xterm fkeys: kf1=\E[11~ kf2=\E[12~ kf3=\E[13~ kf4=\E[14~  kf5=\E[15~
-                   kf6=\E[17~ kf7=\E[18~ kf8=\E[19~ kf9=\E[20~ kf10=\E[21~ kf11=\E[23~ kf12=\E[24~ */
+	/* xterm fkeys: kf1=\E[11~ kf2=\E[12~ kf3=\E[13~ kf4=\E[14~  kf5=\E[15~
+						 kf6=\E[17~ kf7=\E[18~ kf8=\E[19~ kf9=\E[20~ kf10=\E[21~ kf11=\E[23~ kf12=\E[24~ */
 
 	key_may_set("\x1b[11~",  NE_KEY_F(1));
 	key_may_set("\x1b[12~",  NE_KEY_F(2));
@@ -356,9 +351,9 @@ void read_key_capabilities(void) {
 	key_may_set("\x1b[23~", NE_KEY_F(11));
 	key_may_set("\x1b[24~", NE_KEY_F(12));
 
-   /* vt100 keys:   k1=\EOP    k2=\EOQ    k3=\EOR    k4=\EOS     k5=\EOt 
+	/* vt100 keys:   k1=\EOP    k2=\EOQ    k3=\EOR    k4=\EOS     k5=\EOt 
                     k6=\EOu    k7=\EOv    k8=\EOl    k9=\EOw    k10=\EOy */
-                    
+
 	key_may_set("\x1bOP",  NE_KEY_F(1));
 	key_may_set("\x1bOQ",  NE_KEY_F(2));
 	key_may_set("\x1bOR",  NE_KEY_F(3));
@@ -370,8 +365,8 @@ void read_key_capabilities(void) {
 	key_may_set("\x1bOw",  NE_KEY_F(9));
 	key_may_set("\x1bOy", NE_KEY_F(10));
 
-   /* pc keys:   k1=\E[[A   k2=\E[[B    k3=\E[[C    k4=\E[[D     k5=\E[[E */
-                    
+	/* pc keys:   k1=\E[[A   k2=\E[[B    k3=\E[[C    k4=\E[[D     k5=\E[[E */
+
 	key_may_set("\x1b[[A",  NE_KEY_F(1));
 	key_may_set("\x1b[[B",  NE_KEY_F(2));
 	key_may_set("\x1b[[C",  NE_KEY_F(3));
@@ -381,10 +376,10 @@ void read_key_capabilities(void) {
 	/* If at this point any sequence of the form ESC+ASCII character is free, we bind
 		it to the simulated META key. */
 
-	for(i = 1; i < 128; i++) key_may_set(meta_prefixed[i], NE_KEY_META(i));
+	for(int i = 1; i < 128; i++) key_may_set(meta_prefixed[i], NE_KEY_META(i));
 
 #ifdef DEBUGPRINTF
-   dump_keys();
+	dump_keys();
 #endif
 }
 
@@ -392,8 +387,8 @@ void read_key_capabilities(void) {
 
 
 /* Sets the escape time, which is an option, but it's global to ne and it's not
-	saved in autopreferences files. However, an EscapeTime command can be
-	attached manually to any preferences file. */
+   saved in autopreferences files. However, an EscapeTime command can be
+   attached manually to any preferences file. */
 
 static int escape_time = 10;
 
@@ -403,8 +398,8 @@ void set_escape_time(const int new_escape_time) {
 
 
 /* Sets the current timeout in the termios structure relative to stdin. If the
-	timeout value (in tenth of a second) is positive, VMIN is set to 0,
-	otherwise to 1. */
+   timeout value (in tenth of a second) is positive, VMIN is set to 0,
+   otherwise to 1. */
 
 static void set_termios_timeout(const int timeout) {
 	struct termios termios;
@@ -419,28 +414,28 @@ static void set_termios_timeout(const int timeout) {
 
 
 /* Reads in characters, and tries to match them with the sequences
-	corresponding to special keys. Returns a positive number, denoting
-	a character (possibly INVALID_CHAR), or a negative number denoting a key
-	code (if x is the key code, -x-1 will be returned).
+   corresponding to special keys. Returns a positive number, denoting
+   a character (possibly INVALID_CHAR), or a negative number denoting a key
+   code (if x is the key code, -x-1 will be returned).
 
-	This function tries to be highly optimized and efficient by employing a
-	sorted array of strings for the terminal keys. An index keeps track of the
-	key which has a partial match with the current contents of the keyboard
-	buffer. As each character is input, a match is tried with the rest of the
-	string. If a new character does not match, we can just increment the key
-	counter (because the array is sorted). When we get out of the array, we give
-	back the first char in the keyboard buffer (the next call will retry a match
-	on the following chars). */
+   This function tries to be highly optimized and efficient by employing a
+   sorted array of strings for the terminal keys. An index keeps track of the
+   key which has a partial match with the current contents of the keyboard
+   buffer. As each character is input, a match is tried with the rest of the
+   string. If a new character does not match, we can just increment the key
+   counter (because the array is sorted). When we get out of the array, we give
+   back the first char in the keyboard buffer (the next call will retry a match
+   on the following chars). */
 
 
 int get_key_code(void) {
-
 	static int cur_len = 0;
-	static unsigned char kbd_buffer[KBD_BUF_SIZE];
+	static char kbd_buffer[KBD_BUF_SIZE];
 
-	int c, e, last_match = 0, cur_key = 0, partial_match = FALSE, partial_is_utf8 = FALSE;
+	int c, e, last_match = 0, cur_key = 0;
+	bool partial_match = false, partial_is_utf8 = false;
 
-	while(TRUE) {
+	while(true) {
 
 		if (cur_len) {
 
@@ -448,8 +443,8 @@ int get_key_code(void) {
 			we have to check. */
 
 			while(last_match < cur_len) {
-				if (last_match == 0 && io_utf8 && kbd_buffer[0] >= 0x80) {
-					partial_is_utf8 = TRUE;
+				if (last_match == 0 && io_utf8 && (unsigned char)kbd_buffer[0] >= 0x80) {
+					partial_is_utf8 = true;
 					last_match++;
 				}
 				else if (partial_is_utf8) { 	/* Our partial match is an UTF-8 sequence. */
@@ -463,41 +458,41 @@ int get_key_code(void) {
 					else {
 						/* A UTF-8 error. We discard the first character and try again. */
 						if (--cur_len) memmove(kbd_buffer, kbd_buffer + 1, cur_len);
-						partial_is_utf8 = FALSE;
+						partial_is_utf8 = false;
 						last_match = 0;
 					}
 				}
 				else {
 					/* First easy case. We felt off the array. We return the first character
 						in the buffer and restart the match. */
-					
+
 					if (!key[cur_key].string) {
 						c = kbd_buffer[0];
 						if (--cur_len) memmove(kbd_buffer, kbd_buffer + 1, cur_len);
 						return c;
 					}
-					
+
 					/* Second case. We have a partial match on the first last_match
 						characters. If another character matches, either the string is terminated,
 						and we return the key code, or we increment the match count. */
-					
-					
+
+
 					else if (key[cur_key].string[last_match] == kbd_buffer[last_match]) {
 						if (key[cur_key].string[last_match + 1] == 0) {
 							if (cur_len -= last_match + 1) memmove(kbd_buffer, kbd_buffer + last_match + 1, cur_len);
-							
+
 							assert(key[cur_key].code < NUM_KEYS);
-							
+
 							return -key[cur_key].code - 1;
 						}
 						else last_match++;
 					}
-					
+
 					/* The tricky part. If there is a failed match, the order guarantees that
 						no match if possible if the code of the keyboard char is greater than the code of
 						the capability char. Otherwise, we check for the first capability starting
 						with the current keyboard characters. */
-					
+
 					else {
 						if (kbd_buffer[last_match] > key[cur_key].string[last_match]) {
 							c = kbd_buffer[0];
@@ -515,10 +510,10 @@ int get_key_code(void) {
 				tenths of second. If nothing arrives, it is probably time to return
 				what we got. Note that this won't work properly if the terminal has
 				a key capability which is a prefix of another key capability. */
-			
-			partial_match = TRUE;
+
+			partial_match = true;
 		}
-		
+
 		fflush(stdout);
 
 		if (partial_match) set_termios_timeout(escape_time);
@@ -534,7 +529,7 @@ int get_key_code(void) {
 
 		if (c == EOF && (!partial_match || e) && e != EINTR) kill(getpid(), SIGTERM);
 
-		partial_match = FALSE;
+		partial_match = false;
 
 		if (c != EOF) {
 			if (cur_len < KBD_BUF_SIZE) kbd_buffer[cur_len++] = c;
