@@ -1,7 +1,7 @@
 /* Extern's for names and abbreviations of all the commands.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2015 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -331,6 +331,9 @@ extern const char RIGHTMARGIN_ABBREV[];
 extern const char SAVE_NAME[];
 extern const char SAVE_ABBREV[];
 
+extern const char SAVEALL_NAME[];
+extern const char SAVEALL_ABBREV[];
+
 extern const char SAVEAS_NAME[];
 extern const char SAVEAS_ABBREV[];
 
