@@ -1,22 +1,22 @@
 /* Navigation functions.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna
-	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1993-1998 Sebastiano Vigna
+   Copyright (C) 1999-2015 Todd M. Lewis and Sebastiano Vigna
 
-	This file is part of ne, the nice editor.
+   This file is part of ne, the nice editor.
 
-	This library is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 3 of the License, or (at your
-	option) any later version.
+   This library is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
-	This library is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-	for more details.
+   This library is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
 
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, see <http://www.gnu.org/licenses/>.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 
 #include "ne.h"
@@ -71,20 +71,17 @@ familiar with this diagram.
 
 
 /* "Resyncs" cur_pos (the current character the cursor is on) with cur_x and
-	win_x. It has to take into account the TAB expansion, and can cause
-	left/right movement in order to properly land on a real character.  x is the
-	offset from the beginning of the line after TAB expansion. resync_pos()
-	assumes that tab_size < columns/2. Note that this function has to be called
-	whenever the cursor is moved to a different line, keeping the x position
-	constant. The only way of avoiding this problem is not supporting TABs,
-	which is of course unacceptable. Note that if x_wanted is TRUE, then the
-	wanted_x position is used rather tham cur_x+win_x. */
+   win_x. It has to take into account the TAB expansion, and can cause
+   left/right movement in order to properly land on a real character.  x is the
+   offset from the beginning of the line after TAB expansion. resync_pos()
+   assumes that tab_size < columns/2. Note that this function has to be called
+   whenever the cursor is moved to a different line, keeping the x position
+   constant. The only way of avoiding this problem is not supporting TABs,
+   which is of course unacceptable. Note that if x_wanted is true, then the
+   wanted_x position is used rather tham cur_x+win_x. */
 
 void resync_pos(buffer * const b) {
-
-	line_desc *ld = b->cur_line_desc;
-	int pos, i, width, last_char_width, x = b->win_x + b->cur_x;
-
+	int64_t x = b->win_x + b->cur_x;
 	if (b->x_wanted) x = b->wanted_x;
 
 	assert(b->opt.tab_size < ne_columns / 2);
@@ -94,6 +91,8 @@ void resync_pos(buffer * const b) {
 		return;
 	}
 
+	line_desc *ld = b->cur_line_desc;
+	int64_t i, pos, width, last_char_width;
 	for(i = pos = width = 0; pos < ld->line_len; pos = next_pos(ld->line, pos, b->encoding), i++) {
 
 		if (ld->line[pos] != '\t') width += (last_char_width = get_char_width(&ld->line[pos], b->encoding));
@@ -225,9 +224,9 @@ int line_down(buffer * const b) {
  */
 
 void keep_cursor_on_screen(buffer * const b) {
-   int shift_right;
 	b->opt.tab_size = min(b->opt.tab_size, max(ne_columns / 2 - 1,1));
-	if (shift_right = b->win_x % b->opt.tab_size) {
+	const int shift_right = b->win_x % b->opt.tab_size;
+	if (shift_right) {
 		b->win_x -= shift_right;
 		b->cur_x += shift_right;
 	}
@@ -239,7 +238,7 @@ void keep_cursor_on_screen(buffer * const b) {
 			b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.next;
 		}
 		assert(b->win_y = b->cur_line - b->cur_y);
-		b->y_wanted = FALSE;
+		b->y_wanted = false;
 	}
 
 	while(b->cur_x >= ne_columns) {
@@ -251,11 +250,11 @@ void keep_cursor_on_screen(buffer * const b) {
 
 
 /* Moves win_x of n bytes to the left (n *has* to be a multiple of the current
-	TAB size). It is used by char_left(). cur_x is moved, too. */
+   TAB size). It is used by char_left(). cur_x is moved, too. */
 
 static void block_left(buffer * const b, const int n) {
 
-	int t = b->win_x;
+	const int64_t t = b->win_x;
 
 	assert(n <= ne_columns);
 	assert(!(n % b->opt.tab_size));
@@ -348,9 +347,6 @@ int char_right(buffer * const b) {
 
 int prev_page(buffer * const b) {
 
-	int i;
-	line_desc *ld_top, *ld_cur;
-
 	b->y_wanted = 0;
 
 	if (b->cur_y > 0) {
@@ -370,10 +366,10 @@ int prev_page(buffer * const b) {
 
 	if ((b->win_y -= ne_lines - 2)<0) b->win_y = 0;
 
-	ld_top = b->top_line_desc;
-	ld_cur = b->cur_line_desc;
+	line_desc *ld_top = b->top_line_desc;
+	line_desc *ld_cur = b->cur_line_desc;
 
-	for(i = 0; i < ne_lines - 2 && ld_top->ld_node.prev->prev; i++) {
+	for(int i = 0; i < ne_lines - 2 && ld_top->ld_node.prev->prev; i++) {
 		ld_top = (line_desc *)ld_top->ld_node.prev;
 		ld_cur = (line_desc *)ld_cur->ld_node.prev;
 		b->cur_line--;
@@ -390,17 +386,16 @@ int prev_page(buffer * const b) {
 
 
 int next_page(buffer * const b) {
-	int i, disp;
-	line_desc *ld_top, *ld_cur;
-
 	b->y_wanted = 0;
 
 	if (b->cur_y < ne_lines - 2) {
 		update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
+		line_desc *ld_cur;
 		if (b->win_y >= b->num_lines - (ne_lines - 1)) {
 			ld_cur = b->top_line_desc;
+			int i;
 			for(i = 0; i < ne_lines - 2 && ld_cur->ld_node.next->next; i++)
-			   ld_cur = (line_desc *)ld_cur->ld_node.next;
+				ld_cur = (line_desc *)ld_cur->ld_node.next;
 			b->cur_line += (i - b->cur_y);
 			b->cur_y = i;
 		}
@@ -408,7 +403,7 @@ int next_page(buffer * const b) {
 			b->cur_line += (ne_lines - 2 - b->cur_y);
 			b->cur_y = ne_lines - 2;
 			ld_cur = b->top_line_desc;
-			for(i = 0; i < ne_lines - 2; i++)
+			for(int i = 0; i < ne_lines - 2; i++)
 			  ld_cur = (line_desc *)ld_cur->ld_node.next;
 		}
 		b->attr_len = -1;
@@ -422,7 +417,7 @@ int next_page(buffer * const b) {
 	update_syntax_states(b, -1, b->cur_line_desc, NULL);
 	b->attr_len = -1;
 
-	disp = ne_lines - 2;
+	int disp = ne_lines - 2;
 
 	if (b->win_y + disp > b->num_lines - (ne_lines - 1))
 		disp = b->num_lines - (ne_lines - 1) - b->win_y;
@@ -430,10 +425,10 @@ int next_page(buffer * const b) {
 	b->win_y += disp;
 	b->cur_line += disp;
 
-	ld_top = b->top_line_desc;
-	ld_cur = b->cur_line_desc;
+	line_desc *ld_top = b->top_line_desc;
+	line_desc *ld_cur = b->cur_line_desc;
 
-	for(i = 0; i < disp && ld_top->ld_node.next->next; i++) {
+	for(int i = 0; i < disp && ld_top->ld_node.next->next; i++) {
 		ld_top = (line_desc *)ld_top->ld_node.next;
 		ld_cur = (line_desc *)ld_cur->ld_node.next;
 	}
@@ -447,34 +442,29 @@ int next_page(buffer * const b) {
 }
 
 int page_up(buffer * const b) {
-
-	int i, disp;
-
-	disp = ne_lines - 2;
-
 	/* Already on the top line? */
 	if (b->cur_line == 0) return OK;
-	
+
 	update_syntax_states(b, -1, b->cur_line_desc, NULL);
 	b->attr_len = -1;
 
 	if (!b->y_wanted) {
-		b->y_wanted = TRUE;
+		b->y_wanted = true;
 		b->wanted_y = b->cur_line;
 		b->wanted_cur_y = b->cur_y;
-   }
+	}
 
-	for (i = 0; i < disp; i++) {
+	for (int i = 0; i < ne_lines - 2; i++) {
 		b->wanted_y--; /* We want to move up */
-			
+
 		/* Can we move up? */
-		if (b->wanted_y >= 0 /* We aren't yet off the top    */
+		if (b->wanted_y >= 0 /* We aren't yet off the top */
 				&& b->wanted_y < b->num_lines - 1) { /* we aren't still past the end */
 			b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.prev;
 			b->cur_line--;
 		}
-		
-	   /* Should we shift the view up? */
+
+		/* Should we shift the view up? */
 		if (b->win_y > 0 /* We aren't already at the top */
 				&& b->win_y + b->wanted_cur_y > b->wanted_y) { /* Gap between virtual cursor and TOS is to small */
 			b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.prev;
@@ -491,36 +481,32 @@ int page_up(buffer * const b) {
 }
 
 int page_down(buffer * const b) {
-
-	int i, disp, shift_view;
-
-	disp = ne_lines - 2;
-
 	/* Already on the bottom line? */
 	if (b->cur_line == b->num_lines - 1) return OK;
-	
+
 	update_syntax_states(b, -1, b->cur_line_desc, NULL);
 	b->attr_len = -1;
 
 	if (!b->y_wanted) {
-		b->y_wanted = TRUE;
+		b->y_wanted = true;
 		b->wanted_y = b->cur_line;
 		b->wanted_cur_y = b->cur_y;
-   }
+	}
 
-   shift_view = (b->win_y + disp < b->num_lines); /* can't already see the last line */
+	const int disp = ne_lines - 2;
+	const int shift_view = (b->win_y + disp < b->num_lines); /* can't already see the last line */
 
-	for (i = 0; i < disp; i++) {
+	for (int i = 0; i < disp; i++) {
 		b->wanted_y++; /* We want to move down */
-			
+
 		/* Can we move down? */
 		if (b->wanted_y > 0 /* We aren't still above the top  */
 				&&  b->wanted_y < b->num_lines) { /* we aren't yet to the end */
 			b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.next;
 			b->cur_line++;
 		}
-		
-	   /* Should we shift the view down? */
+
+		/* Should we shift the view down? */
 		if (shift_view /* already decided we should */
 				&& b->wanted_y - b->wanted_cur_y >  b->win_y) { /* Gap between virtual cursor and TOS is to big */
 			b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.next;
@@ -552,18 +538,18 @@ int move_tos(buffer * const b) {
 }
 
 int move_bos(buffer * const b) {
-	int i;
-	line_desc *ld_cur;
 
 	b->y_wanted = 0;
 
 	if (b->cur_y < ne_lines - 2) {
 		update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
 		b->attr_len = -1;
+		line_desc *ld_cur;
 		if (b->win_y >= b->num_lines - (ne_lines - 1)) {
 			ld_cur = b->top_line_desc;
-			for(i = 0; i < ne_lines - 2  &&   ld_cur->ld_node.next->next; i++)
-			   ld_cur = (line_desc *)ld_cur->ld_node.next;
+			int i;
+			for(i = 0; i < ne_lines - 2  &&	ld_cur->ld_node.next->next; i++)
+				ld_cur = (line_desc *)ld_cur->ld_node.next;
 			b->cur_line += (i - b->cur_y);
 			b->cur_y = i;
 		}
@@ -571,7 +557,7 @@ int move_bos(buffer * const b) {
 			b->cur_line += (ne_lines - 2 - b->cur_y);
 			b->cur_y = ne_lines - 2;
 			ld_cur = b->top_line_desc;
-			for(i = 0; i < ne_lines - 2; i++)
+			for(int i = 0; i < ne_lines - 2; i++)
 			  ld_cur = (line_desc *)ld_cur->ld_node.next;
 		}
 		b->cur_line_desc = ld_cur;
@@ -584,17 +570,16 @@ int move_bos(buffer * const b) {
    win_x, cur_x, win_y and cur_y -- the variables which control which part
    of the file is visible in the terminal window. */
 
-int adjust_view(buffer * const b, const unsigned char *p) {
-	int i, disp, mag, rc = OK;
-	char *q;
-
+int adjust_view(buffer * const b, const char *p) {
 	b->y_wanted = 0;
 
 	if (!p) p = "t";
-	
+	int rc = OK;
+
 	while(*p) {
-		disp = 0;
-		mag = max(0,strtol(p+1, &q, 0));
+		int disp = 0;
+		char *q;
+		int mag = max(0,strtol(p+1, &q, 0));
 		switch (*p) {
 			case 't' :
 			case 'T' :
@@ -627,15 +612,15 @@ int adjust_view(buffer * const b, const unsigned char *p) {
 
 			case 'c' :
 			case 'C' :
-			   /* Shift the view as far left as possible. This way we don't have to
-			      deal with figuring out which side of Middle the view started on. */
+				/* Shift the view as far left as possible. This way we don't have to
+					deal with figuring out which side of Middle the view started on. */
 				while (b->cur_x >= b->opt.tab_size) {
 					b->win_x += b->opt.tab_size;
 					b->cur_x -= b->opt.tab_size;
 				}
 				/* Since we now know that the cursor is left of center, we can start
-				   to shift the view right until the cursor is centered or until we
-				   run out of text to shift right. */
+					to shift the view right until the cursor is centered or until we
+					run out of text to shift right. */
 				while (b->cur_x < (ne_columns / 2) - (ne_columns / 2) % b->opt.tab_size && b->win_x >= b->opt.tab_size) {
 					b->win_x -= b->opt.tab_size;
 					b->cur_x += b->opt.tab_size;
@@ -655,19 +640,19 @@ int adjust_view(buffer * const b, const unsigned char *p) {
 
 			default  :
 				/* When we hit a character we don't recognize, we set the rc, but
-				   we still process other valid view displacements. */
+					we still process other valid view displacements. */
 				rc = ERROR;
 				break;
 		}
 		if (disp > 0) {
-			for(i = 0; i < disp && b->top_line_desc->ld_node.prev->prev; i++) {
+			for(int i = 0; i < disp && b->top_line_desc->ld_node.prev->prev; i++) {
 				b->win_y--;
 				b->cur_y++;
 				b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.prev;
 			}
 		}
 		else if (disp < 0) {
-			for(i = 0; i > disp && b->top_line_desc->ld_node.next->next; i--) {
+			for(int i = 0; i > disp && b->top_line_desc->ld_node.next->next; i--) {
 				b->win_y++;
 				b->cur_y--;
 				b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.next;
@@ -683,21 +668,20 @@ int adjust_view(buffer * const b, const unsigned char *p) {
 
 
 
-void goto_line(buffer * const b, const int n) {
-	int i;
-	line_desc *ld;
-
+void goto_line(buffer * const b, const int64_t n) {
 	b->y_wanted = 0;
 
 	if (n >= b->num_lines || n == b->cur_line) return;
 
+	line_desc *ld;
+
 	if (n >= b->win_y && n < b->win_y + ne_lines - 1) {
 		update_syntax_states(b, -1, b->cur_line_desc, NULL);
 		b->attr_len = -1;
 		b->cur_y = n - b->win_y;
 		b->cur_line = n;
 		ld = b->top_line_desc;
-		for(i = 0; i < b->cur_y; i++) ld = (line_desc *)ld->ld_node.next;
+		for(int64_t i = 0; i < b->cur_y; i++) ld = (line_desc *)ld->ld_node.next;
 		b->cur_line_desc = ld;
 		resync_pos(b);
 		return;
@@ -712,20 +696,13 @@ void goto_line(buffer * const b, const int n) {
 	if (b->win_y < 0) b->win_y = 0;
 
 	b->cur_y = n - b->win_y;
+
+	ld = nth_line_desc(b, n);
+
 	b->cur_line = n;
-
-	if (n < b->num_lines / 2) {
-		ld = (line_desc *)b->line_desc_list.head;
-		for(i = 0; i < b->win_y; i++) ld = (line_desc *)ld->ld_node.next;
-	}
-	else {
-		ld = (line_desc *)b->line_desc_list.tail_pred;
-		for(i = b->num_lines - 1; i > b->win_y; i--) ld = (line_desc *)ld->ld_node.prev;
-	}
-
-	b->top_line_desc = ld;
-	for(i = 0; i < b->cur_y; i++) ld = (line_desc *)ld->ld_node.next;
 	b->cur_line_desc = ld;
+	for(int i = 0; i < b->cur_y; i++) ld = (line_desc *)ld->ld_node.prev;
+	b->top_line_desc = ld;
 
 	if (b == cur_buffer) update_window(b);
 	resync_pos(b);
@@ -733,7 +710,7 @@ void goto_line(buffer * const b, const int n) {
 
 
 
-void goto_column(buffer * const b, const int n) {
+void goto_column(buffer * const b, const int64_t n) {
 
 	b->x_wanted = 0;
 	b->y_wanted = 0;
@@ -759,7 +736,7 @@ void goto_column(buffer * const b, const int n) {
 /* This is like a goto_column(), but you specify a position (i.e.,
 a character offset) instead. */
 
-void goto_pos(buffer * const b, const int pos) {
+void goto_pos(buffer * const b, const int64_t pos) {
 	goto_column(b, calc_width(b->cur_line_desc, pos, b->opt.tab_size, b->encoding));
 }
 
@@ -767,23 +744,20 @@ void goto_pos(buffer * const b, const int pos) {
 
 void move_to_sol(buffer * const b) {
 
-	int t;
-
 	b->x_wanted = 0;
 	b->y_wanted = 0;
 
-	t = b->win_x;
+	const bool update = b->win_x && b == cur_buffer;
 	b->win_x =
 	b->cur_x =
 	b->cur_pos =
 	b->cur_char = 0;
 
-	if (t &&  b == cur_buffer) update_window(b);
+	if (update) update_window(b);
 }
 
 
 void move_to_eol(buffer * const b) {
-	int i, pos, width, total_width;
 	line_desc *ld = b->cur_line_desc;
 
 	assert(ld->ld_node.next != NULL);
@@ -797,7 +771,7 @@ void move_to_eol(buffer * const b) {
 		return;
 	}
 
-	total_width = calc_width(ld, ld->line_len, b->opt.tab_size, b->encoding);
+	const int64_t total_width = calc_width(ld, ld->line_len, b->opt.tab_size, b->encoding);
 
 	if (total_width >= b->win_x && total_width < b->win_x + ne_columns) {
 		/* We move to a visible position. */
@@ -807,12 +781,12 @@ void move_to_eol(buffer * const b) {
 		return;
 	}
 
-	for(i = pos = width = 0; pos < ld->line_len; pos = next_pos(ld->line, pos, b->encoding), i++)  {
+	for(int64_t i = 0, pos = 0, width = 0; pos < ld->line_len; pos = next_pos(ld->line, pos, b->encoding), i++)  {
 		if (ld->line[pos] != '\t') width += get_char_width(&ld->line[pos], b->encoding);
 		else width += b->opt.tab_size - width % b->opt.tab_size;
 
 		if (total_width - width < ne_columns - b->opt.tab_size) {
-			int t = b->win_x;
+			int64_t t = b->win_x;
 			b->win_x = width - width % b->opt.tab_size;
 			b->cur_x = total_width - b->win_x;
 			b->cur_pos = ld->line_len;
@@ -822,13 +796,13 @@ void move_to_eol(buffer * const b) {
 		}
 	}
 
-	assert(FALSE);
+	assert(false);
 }
 
 
 
 /* Sets the variables like a move_to_sof(), but does not perform any
-	update. This is required in several places. */
+   update. This is required in several places. */
 
 void reset_position_to_sof(buffer * const b) {
 	b->x_wanted =
@@ -847,7 +821,7 @@ void reset_position_to_sof(buffer * const b) {
 
 
 void move_to_sof(buffer * const b) {
-	int moving = b->win_x || b->win_y;
+	const bool moving = b->win_x || b->win_y;
 
 	if (moving) update_syntax_states(b, -1, b->cur_line_desc, NULL);
 	else update_syntax_states(b, b->cur_y, b->cur_line_desc, NULL);
@@ -859,15 +833,14 @@ void move_to_sof(buffer * const b) {
 
 
 void move_to_bof(buffer * const b) {
-	int i;
-	int old_win_x = b->win_x, old_win_y = b->win_y;
+
 	line_desc *ld = (line_desc *)b->line_desc_list.tail_pred;
+	for(int i = 0; i < ne_lines - 2 && ld->ld_node.prev->prev; i++) ld = (line_desc *)ld->ld_node.prev;
 
 	b->x_wanted = 0;
 	b->y_wanted = 0;
 
-	for(i = 0; i < ne_lines - 2 && ld->ld_node.prev->prev; i++) ld = (line_desc *)ld->ld_node.prev;
-
+	const int64_t old_win_x = b->win_x, old_win_y = b->win_y;
 	b->cur_line = b->num_lines - 1;
 	b->win_x = 0;
 	b->win_y = ld->ld_node.prev->prev ? b->num_lines - (ne_lines - 1) : b->num_lines - 1;
@@ -921,33 +894,29 @@ void toggle_sol_eol(buffer * const b) {
 
 
 /* Searches for the start of the next or previous word, depending on the value
-	of dir. */
+   of dir. */
 
 int search_word(buffer * const b, const int dir) {
-
-	line_desc *ld;
-	int c, y, pos = b->cur_pos, word_started = FALSE, space_skipped = FALSE;
-
 	assert(dir == -1 || dir == 1);
 
-	ld = b->cur_line_desc;
+	line_desc *ld = b->cur_line_desc;
+
+	int64_t pos = b->cur_pos;
+	bool word_started = false, space_skipped = false;
 
 	if (pos >= ld->line_len) pos = ld->line_len;
-	else {
-		c = get_char(&ld->line[pos], b->encoding);
-		if (!ne_isword(c, b->encoding)) space_skipped = TRUE;
-	}
+	else if (!ne_isword(get_char(&ld->line[pos], b->encoding), b->encoding)) space_skipped = true;
 
 	if (dir < 0 || pos < ld->line_len)
 		pos = (dir > 0 ? next_pos : prev_pos)(ld->line, pos, b->encoding);
 
-	y = b->cur_line;
+	int64_t y = b->cur_line;
 
 	while(y < b->num_lines && y >= 0) {
 		while(pos < ld->line_len && pos >= 0) {
-			c = get_char(&ld->line[pos], b->encoding);
-			if (!ne_isword(c, b->encoding)) space_skipped = TRUE;
-			else word_started = TRUE;
+			const int c = get_char(&ld->line[pos], b->encoding);
+			if (!ne_isword(c, b->encoding)) space_skipped = true;
+			else word_started = true;
 
 			if (dir > 0) {
 				if (space_skipped && ne_isword(c, b->encoding)) {
@@ -973,18 +942,18 @@ int search_word(buffer * const b, const int dir) {
 			pos = (dir > 0 ? next_pos : prev_pos)(ld->line, pos, b->encoding);
 		}
 
-		space_skipped = TRUE;
+		space_skipped = true;
 
 		if (dir > 0) {
-         ld = (line_desc *)ld->ld_node.next;
-         y++;
+			ld = (line_desc *)ld->ld_node.next;
+			y++;
 			pos = 0;
-      }
-      else {
-         ld = (line_desc *)ld->ld_node.prev;
-         y--;
+		}
+		else {
+			ld = (line_desc *)ld->ld_node.prev;
+			y--;
 			if (ld->ld_node.prev) pos = prev_pos(ld->line, ld->line_len, b->encoding);
-      }
+		}
 	}
 	return ERROR;
 }
@@ -992,29 +961,25 @@ int search_word(buffer * const b, const int dir) {
 
 
 /* Moves to the character after the end of the current word. It doesn't move at
-	all on US-ASCII spaces and punctuation. */
+   all on US-ASCII spaces and punctuation. */
 
 void move_to_eow(buffer * const b) {
 
 	line_desc *ld = b->cur_line_desc;
-   int pos = b->cur_pos, c;
+	int64_t pos = b->cur_pos;
+	if (pos >= ld->line_len || !ne_isword(get_char(&ld->line[pos], b->encoding), b->encoding)) return;
 
-	if (pos >= ld->line_len || !ne_isword(c = get_char(&ld->line[pos], b->encoding), b->encoding)) return;
-
-	while(pos < ld->line_len) {
-		c = get_char(&ld->line[pos], b->encoding);
-		if (!ne_isword(c, b->encoding)) break;
-		pos += b->encoding == ENC_UTF8 ? utf8len(ld->line[pos]) : 1;
-	}
+	for(pos = b->cur_pos; pos < ld->line_len; pos += b->encoding == ENC_UTF8 ? utf8len(ld->line[pos]) : 1) 
+		if (!ne_isword(get_char(&ld->line[pos], b->encoding), b->encoding)) break;
 
 	goto_pos(b, pos);
 }
 
 
 /* Implements Brief's "incrementale move to the end": if we are in the middle
-	of a line, we mode to the end of line; otherwise, if we are in the middle of
-	a page, we move to the end of the page; otherwise, if we are in the middle
-	of a file we move to the end of file. */
+   of a line, we mode to the end of line; otherwise, if we are in the middle of
+   a page, we move to the end of the page; otherwise, if we are in the middle
+   of a file we move to the end of file. */
 
 void move_inc_down(buffer * const b) {
 
