@@ -1,22 +1,22 @@
 /* Preferences functions.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1993-1998 Sebastiano Vigna 
+   Copyright (C) 1999-2015 Todd M. Lewis and Sebastiano Vigna
 
-	This file is part of ne, the nice editor.
+   This file is part of ne, the nice editor.
 
-	This library is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 3 of the License, or (at your
-	option) any later version.
+   This library is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
-	This library is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-	for more details.
+   This library is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
 
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, see <http://www.gnu.org/licenses/>.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 
 #include "ne.h"
@@ -41,21 +41,19 @@ bigger just causes a reallocation. */
 #define PREF_FILE_SIZE_GUESS 256
 
 /* If we're saving default prefs, we include global prefs
-	that are not buffer specific. Likewise, if we're saving
-	auto prefs, we don't want to include global prefs. */
+   that are not buffer specific. Likewise, if we're saving
+   auto prefs, we don't want to include global prefs. */
 
-static int saving_global;
+static bool saving_global;
 
 /* Returns a pointer to the extension of a filename, or NULL if there is no
    extension. Note that filename has to be non NULL. */
 
 const char *extension(const char * const filename) {
 
-	int i;
-
 	assert(filename != NULL);
 
-	for(i = strlen(filename); i-- != 0;)
+	for(int i = strlen(filename); i-- != 0;)
 		if (filename[i] == '.') return &filename[i + 1];
 
 	return NULL;
@@ -72,8 +70,6 @@ const char *extension(const char * const filename) {
 char *exists_prefs_dir(void) {
 
 	static char *prefs_dir;
-	char *home_dir;
-	struct stat s;
 
 	/* If we have been already called, we already computed the name. */
 
@@ -82,21 +78,23 @@ char *exists_prefs_dir(void) {
 	/* In the UN*X case, we first get the home directory. Then
 	we allocate space for the directory name. */
 
+	char * home_dir;
 	if (!(home_dir = getenv("HOME"))) home_dir = ".";
 
 	if (prefs_dir = malloc(strlen(home_dir) + strlen(PREFS_DIR) + 3)) {
 
 		strcat(strcat(strcpy(prefs_dir, home_dir), "/"), PREFS_DIR);
 
-	   if (stat(prefs_dir, &s)) {
-   	   if (mkdir(prefs_dir, 0700)) {
+		struct stat s;
+		if (stat(prefs_dir, &s)) {
+			if (mkdir(prefs_dir, 0700)) {
 				free(prefs_dir);
-   	   	return prefs_dir = NULL;
+				return prefs_dir = NULL;
 			}
 		}
-	   else if (!S_ISDIR(s.st_mode)) {
+		else if (!S_ISDIR(s.st_mode)) {
 			free(prefs_dir);
-	   	return prefs_dir = NULL;
+			return prefs_dir = NULL;
 		}
 
 		return strcat(prefs_dir, "/");
@@ -111,20 +109,20 @@ char *exists_prefs_dir(void) {
 
 char *exists_gprefs_dir(void) {
 	static char *gprefs_dir = NULL;
-	struct stat s;
-	const char *global_dir;
 
 	/* If we have been already called, we already computed the name. We
 		should free up the name and re-compute it (because the global dir may
 		have changed). */
-    
+
 	if (gprefs_dir) {
 		free(gprefs_dir);
 		gprefs_dir = NULL;
 	}
 
+	const char *global_dir;
 	if ((global_dir = get_global_dir()) && (gprefs_dir = malloc(strlen(global_dir) + 3 ))) {
 		strcpy(gprefs_dir, global_dir);
+		struct stat s;
 		if (stat(gprefs_dir, &s)) {
 			free(gprefs_dir);
 			return gprefs_dir = NULL;
@@ -143,18 +141,15 @@ char *exists_gprefs_dir(void) {
    name are NULL, ERROR is returned.  */
 
 int save_prefs(buffer * const b, const char * const name) {
-
-	int error;
-	char_stream *cs;
-
 	if (!b || !name) return ERROR;
 
 	assert_buffer(b);
 
-	if (cs = alloc_char_stream(PREF_FILE_SIZE_GUESS)) {
+	char_stream *cs = alloc_char_stream(PREF_FILE_SIZE_GUESS);
+	if (cs) {
 		/* We create a macro by recording an action for each kind of flag. */
 
-		if (!saving_global && b->syn) record_action(cs, SYNTAX_A, -1, b->syn->name, verbose_macros);
+		if (!saving_global && b->syn) record_action(cs, SYNTAX_A, -1, (const char *)b->syn->name, verbose_macros);
 
 		record_action(cs, TABSIZE_A,          b->opt.tab_size,       NULL, verbose_macros);
 		record_action(cs, CLIPNUMBER_A,       b->opt.cur_clip,       NULL, verbose_macros);
@@ -168,8 +163,8 @@ int save_prefs(buffer * const b, const char * const name) {
 		record_action(cs, DOUNDO_A,           b->opt.do_undo,        NULL, verbose_macros);
 		record_action(cs, AUTOPREFS_A,        b->opt.auto_prefs,     NULL, verbose_macros);
 		record_action(cs, NOFILEREQ_A,        b->opt.no_file_req,    NULL, verbose_macros);
-      /* Skip read_only */
-      /* Skip search_back */
+	   /* Skip read_only */
+	   /* Skip search_back */
 		record_action(cs, CASESEARCH_A,       b->opt.case_search,    NULL, verbose_macros);
 		record_action(cs, TABS_A,             b->opt.tabs,           NULL, verbose_macros);
 		record_action(cs, DELTABS_A,          b->opt.del_tabs,       NULL, verbose_macros);
@@ -178,21 +173,23 @@ int save_prefs(buffer * const b, const char * const name) {
 		record_action(cs, BINARY_A,           b->opt.binary,         NULL, verbose_macros);
 		record_action(cs, UTF8AUTO_A,         b->opt.utf8auto,       NULL, verbose_macros);
 		record_action(cs, VISUALBELL_A,       b->opt.visual_bell,    NULL, verbose_macros);
-		
+
 		if (saving_global) {
 			/* We only save the global flags that differ from their defaults. */
 			/* Make sure these are in sync with the defaults near the top of ne.c. */
+#ifndef ALTPAGING
 			if (req_order)       record_action(cs, REQUESTORDER_A,  req_order,      NULL, verbose_macros);
+#else
+			if (!req_order)      record_action(cs, REQUESTORDER_A,  req_order,      NULL, verbose_macros);
+#endif
 			if (fast_gui)        record_action(cs, FASTGUI_A,       fast_gui,       NULL, verbose_macros);
 			if (!status_bar)     record_action(cs, STATUSBAR_A,     status_bar,     NULL, verbose_macros);
 			if (!verbose_macros) record_action(cs, VERBOSEMACROS_A, verbose_macros, NULL, verbose_macros);
-			saving_global = 0;
+			saving_global = false;
 		}
 
-		error = save_stream(cs, name, b->is_CRLF, FALSE);
-
+		const int error = save_stream(cs, name, b->is_CRLF, false);
 		free_char_stream(cs);
-
 		return error;
 	}
 
@@ -205,17 +202,15 @@ int save_prefs(buffer * const b, const char * const name) {
    exec_only_options flag set. If b or name are NULL, ERROR is returned. */
 
 int load_prefs(buffer * const b, const char * const name) {
-
-	int error = OK;
-	char_stream *cs;
-
 	if (!b || !name) return ERROR;
 
 	assert_buffer(b);
 
 	b->exec_only_options = 1;
 
-	if (cs = load_stream(NULL, name, FALSE, FALSE)) {
+	int error = OK;
+	char_stream * const cs = load_stream(NULL, name, false, false);
+	if (cs) {
 		error = play_macro(b, cs);
 		free_char_stream(cs);
 	}
@@ -230,11 +225,11 @@ int load_prefs(buffer * const b, const char * const name) {
    syntax if the new one cannot be loaded. */
 
 int load_syntax_by_name(buffer * const b, const char * const name) {
-	struct high_syntax *syn;
 	assert_buffer(b);
 	assert(name != NULL);
-	syn = load_syntax((char *)name);
-	if (!syn) syn = load_syntax((char *)ext2syntax(name));
+
+	struct high_syntax *syn = load_syntax((unsigned char *)name);
+	if (!syn) syn = load_syntax((unsigned char *)ext2syntax(name));
 	if (syn) {
 		b->syn = syn;
 		reset_syntax_states(b);
@@ -252,10 +247,6 @@ int load_syntax_by_name(buffer * const b, const char * const name) {
    returned.  */
 
 static int do_auto_prefs(buffer *b, const char * ext, int (prefs_func)(buffer *, const char *)) {
-
-	int error = OK;
-	char *auto_name, *prefs_dir;
-
 	if (!b) return ERROR;
 
 	assert_buffer(b);
@@ -266,6 +257,8 @@ static int do_auto_prefs(buffer *b, const char * ext, int (prefs_func)(buffer *,
 	That way the user can override whatever he wants, but anything he
 	doesn't override still gets passed through. */
 
+	int error = OK;
+	char *auto_name, *prefs_dir;
 	if (*prefs_func == load_prefs && (prefs_dir = exists_gprefs_dir())) {
 		if (auto_name = malloc(strlen(ext) + strlen(prefs_dir) + strlen(PREF_FILE_SUFFIX) + 2)) {
 			strcat(strcat(strcpy(auto_name, prefs_dir), ext), PREF_FILE_SUFFIX);
@@ -304,17 +297,15 @@ int load_auto_prefs(buffer * const b, const char *name) {
 
 int save_auto_prefs(buffer * const b, const char *name) {
 	/* In practice, the only time we call save_auto_prefs with a name is
-	   when we save the default prefs. If that changes, so too must this
-	   method of setting this static flag used by save_prefs. */
-	saving_global = name ? 1 : 0;
+		when we save the default prefs. If that changes, so too must this
+		method of setting this static flag used by save_prefs. */
+	saving_global = name ? true : false;
 	return do_auto_prefs(b, name, save_prefs);
 }
 
 
-/**************************************
-  This bit has to do with pushing and poping preferences
-  on the prefs stack.
-****************************************/
+/* This bit has to do with pushing and popping preferences
+   on the prefs stack. */
 
 #define MAX_PREF_STACK_SIZE  32
 
@@ -323,7 +314,7 @@ typedef struct {
 	int psize;
 	options_t pref[MAX_PREF_STACK_SIZE];
 } pref_stack_t;
-  
+
 static pref_stack_t pstack = { 0, MAX_PREF_STACK_SIZE };
 
 int push_prefs(buffer * const b) {
@@ -339,7 +330,7 @@ int push_prefs(buffer * const b) {
 	print_message(msg);
 	return OK;
 }
-  
+
 int pop_prefs(buffer * const b) {
 	char msg[120];
 	if (pstack.pcount <= 0) {
