@@ -1,22 +1,22 @@
 /* Syntax highlighting from Joe's Own Editor: Syntax highlighting DFA interpreter.
 
-	Copyright (C) 2004 Joseph H. Allen
-	Copyright (C) 2009-2013 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 2004 Joseph H. Allen
+   Copyright (C) 2009-2015 Todd M. Lewis and Sebastiano Vigna
 
-	This file is part of ne, the nice editor.
+   This file is part of ne, the nice editor.
 
-	This library is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 3 of the License, or (at your
-	option) any later version.
+   This library is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
-	This library is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-	for more details.
+   This library is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
 
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, see <http://www.gnu.org/licenses/>.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 
 #include "ne.h"
@@ -26,7 +26,7 @@
 
 /* Convert color/attribute name into internal code */
 
-int meta_color_single(unsigned char *s)
+uint32_t meta_color_single(unsigned char *s)
 {
 	if(!zcmp(s,USTR "inverse"))
 		return INVERSE;
@@ -108,7 +108,7 @@ int meta_color_single(unsigned char *s)
 		return BG_BBLACK;
 
 	/* Look at the "256colres.pl" PERL script in the xterm source
-	   distribution to see how these work. */
+		distribution to see how these work. */
 
 	/* 256 color xterm support: bg_RGB and fg_RGB, where R, G, and B range from 0 - 5 */
 	/* Codes 16 - 231 are a 6x6x6 color cube */
@@ -116,13 +116,13 @@ int meta_color_single(unsigned char *s)
 		s[3]>='0' && s[3]<='5' &&
 		s[4]>='0' && s[4]<='5' &&
 		s[5]>='0' && s[5]<='5' && !s[6])
-	        return FG_NOT_DEFAULT | ((16 + (s[3]-'0')*6*6 + (s[4]-'0')*6 + (s[5]-'0')) << FG_SHIFT);
+			  return FG_NOT_DEFAULT | ((16 + (s[3]-'0')*6*6 + (s[4]-'0')*6 + (s[5]-'0')) << FG_SHIFT);
 
 	else if(s[0]=='b' && s[1]=='g' && s[2]=='_' &&
 		  s[3]>='0' && s[3]<='5' &&
 		  s[4]>='0' && s[4]<='5' &&
 		  s[5]>='0' && s[5]<='5' && !s[6])
-	        return BG_NOT_DEFAULT | ((16 + (s[3]-'0')*6*6 + (s[4]-'0')*6 + (s[5]-'0')) << BG_SHIFT);
+			  return BG_NOT_DEFAULT | ((16 + (s[3]-'0')*6*6 + (s[4]-'0')*6 + (s[5]-'0')) << BG_SHIFT);
 
 	/* 256 color xterm support: shades of grey */
 	/* Codes 232 - 255 are shades of grey */
@@ -136,9 +136,9 @@ int meta_color_single(unsigned char *s)
 		return 0;
 }
 
-int meta_color(unsigned char *s)
+uint32_t meta_color(unsigned char *s)
 {
-	int code = 0;
+	uint32_t code = 0;
 	while (*s) {
 		unsigned char buf[32];
 		int x = 0;
@@ -170,48 +170,48 @@ unsigned char *lowerize(unsigned char *s) {
    Global array 'attr_buf' end up with coloring for each character of line (attr_len characters).
    'state' is initial parser state for the line (0 is initial state). */
 
-int *attr_buf = 0;	
-int attr_size = 0;
-int attr_len = 0;
+uint32_t *attr_buf = 0;	
+int64_t attr_size = 0;
+int64_t attr_len = 0;
 int stack_count = 0;
 
-HIGHLIGHT_STATE parse(struct high_syntax *syntax, line_desc *ld, HIGHLIGHT_STATE h_state, int utf8)
+HIGHLIGHT_STATE parse(struct high_syntax * const syntax, line_desc * const ld, HIGHLIGHT_STATE h_state, const bool utf8)
 {
 	struct high_frame *stack = h_state.stack;
-	
+
 	struct high_state *h = (stack ? stack->syntax : syntax)->states[h_state.state];
-	
+
 			/* Current state */
-	
+
 	unsigned char buf[24];			/* Name buffer (trunc after 23 characters) */
 	unsigned char lbuf[24];			/* Lower case version of name buffer */
 	unsigned char lsaved_s[24];		/* Lower case version of delimiter match buffer */
 	int buf_idx=0;				/* Index into buffer */
 	int c;					/* Current character */
 	int c_len;				/* Character length in bytes */
-	int *attr = attr_buf;
-	int *attr_end = attr_buf+attr_size;
+	uint32_t *attr = attr_buf;
+	uint32_t *attr_end = attr_buf+attr_size;
 	int buf_en = 0;				/* Set for name buffering */
 	int ofst = 0;				/* record offset after we've stopped buffering */
 	int mark1 = 0;  			/* offset to mark start from current pos */
 	int mark2 = 0;  			/* offset to mark end from current pos */
 	int mark_en = 0;			/* set if marking */
 	int recolor_delimiter_or_keyword;
-	
-	unsigned char *p = ld->line;			
-	unsigned char *q = ld->line  + ld->line_len;
+
+	const unsigned char *p = (const unsigned char *)ld->line;
+	unsigned char *q = (unsigned char *)(ld->line  + ld->line_len);
 
 	buf[0]=0;				/* Forgot this originally... took 5 months to fix! */
-	
-		
+
+
 	/* Get next character */
-	                  /* Una iterazione in più: aggiungo '\n' come ultimo carattere. */
+							/* Una iterazione in più: aggiungo '\n' come ultimo carattere. */
 	while( p <= q ) { /* On the last itteration, process the virtual '\n' character. */
 		struct high_cmd *cmd, *kw_cmd;
 		int x;
-			
+
 		if (p == q) c = '\n';
-		else c = utf8 ? get_char(p, ENC_UTF8) : *p;
+		else c = utf8 ? get_char((const char*)p, ENC_UTF8) : *p;
 
 		c_len = utf8 ? utf8seqlen(c) : 1;
 		p += c_len;	
@@ -221,8 +221,8 @@ HIGHLIGHT_STATE parse(struct high_syntax *syntax, line_desc *ld, HIGHLIGHT_STATE
 			c = 0x1F;
 
 		/* Create or expand attribute array if necessary */
-		
-		
+
+
 		if(attr==attr_end) {
 			if(!attr_buf) {
 				attr_size = 1024;
@@ -691,14 +691,14 @@ struct high_state *load_dfa(struct high_syntax *syntax)
 	int inside_subr = 0;
 
 	/* Load it */
-	
-	if ((p = exists_prefs_dir()) && strlen(p) + 2 + strlen(SYNTAX_DIR) + strlen(SYNTAX_EXT) + strlen(syntax->name) < sizeof name) {
-		strcat(strcat(strcat(strcat(strcpy(name, p), SYNTAX_DIR), "/"), syntax->name), SYNTAX_EXT);
+
+	if ((p = (unsigned char *)exists_prefs_dir()) && strlen((const char *)p) + 2 + strlen(SYNTAX_DIR) + strlen(SYNTAX_EXT) + strlen((const char *)syntax->name) < sizeof name) {
+		strcat(strcat(strcat(strcat(strcpy((char *)name, (const char *)p), SYNTAX_DIR), "/"), (const char *)syntax->name), SYNTAX_EXT);
 		f = fopen((char *)name,"r");
 	}
 
-	if (!f && (p = exists_gprefs_dir()) && strlen(p) + 2 + strlen(SYNTAX_DIR) + strlen(SYNTAX_EXT) + strlen(syntax->name) < sizeof name) {
-		strcat(strcat(strcat(strcat(strcpy(name, p), SYNTAX_DIR), "/"), syntax->name), SYNTAX_EXT);
+	if (!f && (p = (unsigned char*)exists_gprefs_dir()) && strlen((const char *)p) + 2 + strlen(SYNTAX_DIR) + strlen(SYNTAX_EXT) + strlen((const char *)syntax->name) < sizeof name) {
+		strcat(strcat(strcat(strcat(strcpy((char *)name, (const char *)p), SYNTAX_DIR), "/"), (const char *)syntax->name), SYNTAX_EXT);
 		f = fopen((char *)name,"r");
 	}
 
