@@ -1,32 +1,33 @@
 /* UTF-8 support.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1993-1998 Sebastiano Vigna 
+   Copyright (C) 1999-2015 Todd M. Lewis and Sebastiano Vigna
 
-	This file is part of ne, the nice editor.
+   This file is part of ne, the nice editor.
 
-	This library is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 3 of the License, or (at your
-	option) any later version.
+   This library is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
-	This library is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-	for more details.
+   This library is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
 
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, see <http://www.gnu.org/licenses/>.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 
 #include <assert.h>
+#include <ctype.h>
 #include "utf8.h"
 
 
 /* Computes the character length of an UTF-8 encoded sequence of bytes. */
 
-int utf8strlen(const unsigned char * const s, const int len) {
-	int i = 0, l = 0;
+int64_t utf8strlen(const char * const s, const int64_t len) {
+	int64_t i = 0, l = 0;
 	while(i < len) {
 		assert(utf8len(s[i]) >= 0);
 		i += utf8len(s[i]);
@@ -49,7 +50,8 @@ int utf8seqlen(const int c) {
 
 /* Return the Unicode characters represented by the given string, or -1 if an error occurs. */
 
-int utf8char(const unsigned char * const s) {
+int utf8char(const char * const ss) {
+	const unsigned char * const s = (const unsigned char *)ss;
 	if (s[0] < 0x80) return s[0];
 	if (s[0] < 0xC0) return -1;
 	if (s[0] < 0xE0) return (s[0] & 0x1F) << 6 | s[1] & 0x3F;
@@ -60,10 +62,12 @@ int utf8char(const unsigned char * const s) {
 }
 
 /* Writes the UTF-8 encoding (at most 6 bytes) of the given character to the
-	given string. Returns the length of the string written. */
+   given string. Returns the length of the string written. */
 
-int utf8str(const int c, unsigned char * const s) {
-   if (c < 0x80) {
+int utf8str(const int c, char * const ss) {
+	unsigned char * const s = (unsigned char *)ss;
+
+	if (c < 0x80) {
 		s[0] = c;
 		return 1;
 	}
@@ -113,9 +117,9 @@ int utf8str(const int c, unsigned char * const s) {
 
 int utf8toupper(const int c) {
 #ifdef NOWCHAR
-   return c < 0x80 ? toupper(c) : c;
+	return c < 0x80 ? toupper(c) : c;
 #else
-   return towupper(c);
+	return towupper(c);
 #endif
 }
 
@@ -124,8 +128,8 @@ int utf8toupper(const int c) {
 
 int utf8tolower(const int c) {
 #ifdef NOWCHAR
-   return c < 0x80 ? tolower(c) : c;
+	return c < 0x80 ? tolower(c) : c;
 #else
-   return towlower(c);
+	return towlower(c);
 #endif
 }
